package biggests.tables;

import java.util.*;

import javax.swing.table.*;

/**
 * <p>Title: Table Sorter for Tables with Discrete Values</p>
 *
 * <p>Description: A table model which allows for column sorting.
 *                 This model implements table sorting for tables
 *                 with String values in the first column and
 *                 <code>char</code> values in the remaining ones.<br/>
 * </p>
 *
 * <p>Copyright:   Copyright (C) 2007  Joana Gonalves
 *                 This program is free software; you can redistribute
 *                 it and/or modify it under the terms of the GNU General
 *                 Public License as published by the Free Software
 *                 Foundation; either version 3 of the License, or
 *                 any later version.
 *
 *                 This program is distributed in the hope that it will
 *                 be useful, but WITHOUT ANY WARRANTY; without even the
 *                 implied warranty of MERCHANTABILITY or FITNESS FOR A
 *                 PARTICULAR PURPOSE.  See the
 *                 <a href="http://www.gnu.org/licenses/gpl.html">
 *                 GNU General Public License</a> for more details.
 * </p>
 *
 * @author Joana Gonalves
 */
public class TableSorterSymbols extends TableSorterVerticalHeader {
    /**
     * Constructs a new <code>TableSorterSymbols</code> model. This model
     * implements table sorting and a table header which writes vertical
     * oriented column names, as wel as sets common superclasses of columns'
     * elements.<br/>
     *
     * @see TableSorterVerticalHeader#TableSorterVerticalHeader()
     */
    public TableSorterSymbols() {
        super();
    }

    /**
     * Constructs a new <code>TableSorterSymbols</code> model from a
     * plain table model. This model implements table sorting and a table
     * header which writes vertical oriented column names, as wel as sets
     * common superclasses of columns' elements.<br/>
     *
     * @param tableModel        the <code>TableModel</code> which is the basic
     *                          data model for the table
     * @see TableSorterVerticalHeader#TableSorterVerticalHeader(TableModel tableModel)
     */
    public TableSorterSymbols(TableModel tableModel) {
        super(tableModel);
    }

    /**
     * Constructs a new <code>TableSorterSymbols</code> model from a plain table
     * model and a given table header. This model implements table sorting and
     * a table header which writes vertical oriented column names,
     * as wel as sets common superclasses of columns' elements.<br/>
     *
     * @param tableModel        the <code>TableModel</code> which is the basic
     *                          data model for the table
     * @param tableHeader       the <code>TableHeader</code> which is the
     *                          header for the table
     * @see TableSorterVerticalHeader#TableSorterVerticalHeader(TableModel tableModel, TableHeader tableHeader)
     */
    public TableSorterSymbols(TableModel tableModel, TableHeader tableHeader) {
        super(tableModel, tableHeader);
    }

    /**
     * Overrides <code>TableSorterValues</code>'s <code>getColumnClass</code>
     * method.<br/>
     * Returns the common superclass of the elements at the column with index
     * <code>column</code>.<br/>
     *
     * @param column   <code>int</code> value: the column being queried
     * @return         the common super<code>Class</code> of the column at
     *                 index <code>column</code>
     * @see            TableSorterValues#getColumnClass(int column)
     */
    public Class<?> getColumnClass(int column) {
        if (column == 0) {
            return (String.class);
        } else {
            return (Character.class);
        }
    }

    /**
     * Overrides <code>TableSorterValues</code>'s <code>getComparator</code>
     * method.<br/>
     * Returns the suitable comparator for the values of column number
     * <code>column</code>.
     *
     * @param column the <code>int</code> value which is the index of the column
     * @return the suitable <code>Comparator</code> for column <code>column</code>
     * @see TableValues#getComparator(int column)
     */
    protected Comparator getComparator(int column) {
        Class columnType = getColumnClass(column);
        Comparator comparator = (Comparator) columnComparators.get(columnType);

        if (comparator != null) {
            return comparator;
        }
        if (Comparable.class.isAssignableFrom(columnType)) {
            return COMPARABLE_COMPARATOR;
        }
        return LEXICAL_COMPARATOR;
    }
}
