package biggests.utils;

import de.schlichtherle.io.File;
import biggests.gui.BiggestsFrame;
import java.net.URL;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JFrame;
import java.io.ObjectOutputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import de.schlichtherle.io.FileInputStream;

/**
 * <p>Title: BiGGEsTS Install</p>
 *
 * <p>Description: Provides methods for installing the necessary
 *                 files for BiGGEsTS to execute properly.</p>
 *
 * <p>Copyright:   Copyright (C) 2008  Joana Gonalves
 *                 This program is free software; you can redistribute
 *                 it and/or modify it under the terms of the GNU General
 *                 Public License as published by the Free Software
 *                 Foundation; either version 3 of the License, or
 *                 any later version.
 *
 *                 This program is distributed in the hope that it will
 *                 be useful, but WITHOUT ANY WARRANTY; without even the
 *                 implied warranty of MERCHANTABILITY or FITNESS FOR A
 *                 PARTICULAR PURPOSE.  See the
 *                 <a href="http://www.gnu.org/licenses/gpl.html">
 *                 GNU General Public License</a> for more details.
 * </p>
 *
 * @author Joana P. Gonalves
 */
public class BiggestsInstall {
    /**
     * Creates a new instance of BiGGEsTS Install.
     */
    public void BiggestsInstall() {

    }

    /**
     * Installs Graphviz dot executable or checks and sets the given
     * dot filepath. Installs GO files.
     *
     * @param args String[] the arguments: only the first argument is
     *        used, if it exists; it is used as the Graphviz dot
     *        executable file path in the filesystem
     */
    public static void main(String [] args) {
        BiggestsInstall bi = new BiggestsInstall();

        if (args != null && args.length != 0) {
            if ((args[0].charAt(0) == '"') &&
                (args[0].charAt(args[0].length()-1) == '"')) {
                args[0] = args[0].substring(1, args[0].length()-2);
            }

            File file = new File(args[0]);
            if (file.exists() && file.isFile()) {
                System.out.println("Graphviz dot is available at: " + file.getAbsolutePath());
                writeDOTPathFile(file);
            }
            else {
                JFrame frame = new JFrame();
                frame.setSize(400, 400);
                frame.setVisible(true);
                int op = JOptionPane.showConfirmDialog(frame,
                                              "The dot path you provided " +
                                             "is not valid.\n" +
                                             "Browse the file system?",
                                             "Invalid path to dot",
                                             JOptionPane.YES_NO_OPTION,
                                             JOptionPane.WARNING_MESSAGE);
                if (op == JOptionPane.YES_OPTION) {
                    JFileChooser fileChooser = new JFileChooser();
                    String currDir= System.getProperty("user.dir");
                    fileChooser.setCurrentDirectory(new File(currDir));
                    int o = fileChooser.showOpenDialog(frame);
                    if (o == JFileChooser.APPROVE_OPTION) {
                        file = new File(fileChooser.
                                        getSelectedFile().getAbsolutePath());
                        frame.dispose();
                        if (file.exists() && file.isFile()) {
                            System.out.println("Graphviz dot is available at: "
                                               + file.getAbsolutePath());
                            writeDOTPathFile(file);
                        }
                        else {
                            System.out.println("No installation of Graphviz dot provided.");
                        }
                    }
                    else {
                        frame.setVisible(false);
                        frame.dispose();
                    }
                }
                else {
                    frame.dispose();
                    System.out.println(
                            "No installation of Graphviz dot provided.");
                }
            }
            try {
                System.out.println("Installing GO files...");
                bi.installGOFiles();
                System.out.println("....done");
            }
            catch (Exception e) {
                System.out.println("Failed to install GO files.");
                e.printStackTrace();
            }
            return;
        }
        else {
            String osName = System.getProperty("os.name").toLowerCase();
            if (!isWindows(osName) &&
                !isMacOS(osName)) {
                System.out.println("No installation of Graphviz dot provided.");
                return;
            }
        }

        try {
            System.out.println("Installing dot...");
            bi.installDOT();
            System.out.println("....done");
        }
        catch(Exception e) {
            System.out.println("Failed to install dot.");
            e.printStackTrace();
        }
        try {
            System.out.println("Installing GO files...");
            bi.installGOFiles();
            System.out.println("....done");
        }
        catch (Exception e) {
            System.out.println("Failed to install GO files.");
            e.printStackTrace();
        }
    }

    /**
     * Sets the path to Graphviz dot file executable. If the path
     * to the Graphviz dot is found on the corresponding file, sets
     * the path to Graphviz dot in BiGGEsTS.
     * If<br/>
     * - Graphviz dot is not installed and<br/>
     * - BiGGEsTS is running on Windows or Mac OS and<br/>
     * - Graphviz dot is available in the jar file as a resource<br/>
     * Then<br/>
     * - installs Graphviz dot<br/>
     * - sets the path to Graphviz dot file<br/>
     */
    public static void initDOT() {
        File dotPathFile = new File("pathToDOT.ptd");
        try {
            if (dotPathFile.exists() && dotPathFile.isFile()) {
                ObjectInputStream ois = new ObjectInputStream(new
                        FileInputStream(dotPathFile));
                String path = (String)ois.readObject();
                ois.close();
                File dot = new File(path);
                if (dot.exists() && dot.isFile()) {
                    BiggestsFrame.setPathToDOT(path);
                    return;
                }
            }
        }
        catch (Exception e) {
            System.out.println("Graphviz dot not found.");
        }
        String osName = System.getProperty("os.name").toLowerCase();
        if (isWindows(osName) || isMacOS(osName)) {
            try {
                System.out.println("Installing dot...");
                installDOT();
                System.out.println("....done");
            }
            catch (Exception e) {
                System.out.println("Graphviz dot was not installed.");
            }
        }
    }

    /**
     * Initializes the path to dot binary file, according
     * to the operating system BiGGEsTS is running on.
     *
     * @throws Exception if Graphviz files cannot be found in
     *         the proper resources location or the Graphviz
     *         dot path in the filesystem (after installation)
     *         cannot be written to a binary file (to be read
     *         later by executions of BiGGEsTS)
     */
    public static void installDOT() throws Exception {
        String osName = System.getProperty("os.name").toLowerCase();
        String dotName;
        String middle;
        String sysDIR;
        String resDIR;

        if (isWindows(osName)) {
            sysDIR = "graphviz-win";
            resDIR = "res/graphviz-win";
            middle = "";
            dotName = "dot.exe";
            installDOTWindowsOrMacOS(sysDIR, resDIR, middle,
                                     dotName, osName);
        }
        else
/*        if (isLinux(osName)) {
            resDIR = "res/graphviz-linux";
            sysDIR = File.separator + "usr";
            middle = File.separator + "bin";
            dotName = "dot";
            installDOTLinux(resDIR, sysDIR,
                            middle, dotName);
        }
        else
 */
        if (isMacOS(osName)) {
            sysDIR = "graphviz-mac";
            resDIR = "res/graphviz-mac";
            middle = File.separator + "MacOS";
            dotName = "dot";
            installDOTWindowsOrMacOS(sysDIR, resDIR, middle,
                                     dotName, osName);
        }
        else {
            System.out.println("Graphviz dot was not installed.");
            return;
        }

        File file = new File(sysDIR + middle + File.separator + dotName);
        writeDOTPathFile(file);
        BiggestsFrame.setPathToDOT(sysDIR + middle +
                                   File.separator + dotName);
    }

    /**
     * Stores the path to Graphviz dot on a binary file, to be read
     * later each time BiGGEsTS is executed.
     *
     * @param file File the file corresponding to Graphviz dot executable
     */
    private static void writeDOTPathFile(File file) {
        try {
            File dotPathFile = new File("pathToDOT.ptd");
            ObjectOutputStream oos = new ObjectOutputStream(
                    new FileOutputStream(dotPathFile));
            oos.writeObject(file.getAbsolutePath());
            oos.close();
        }
        catch(Exception e) {
            System.out.println("Error retrieving dot file.");
        }
    }

    /**
     * Checks if osName corresponds to Windows.
     * Assumes that osName was obtained via
     * System.getProperty("os.name") and is lowercase.
     *
     * @param osName String the name of the operating system
     *        as returned by System.getProperty("os.name") method
     *        and set to lowercase
     * @return boolean <code>true</code> if osName identifies
     *         Windows operating system, <code>false</code> otherwise
     */
    private static boolean isWindows(String osName) {
        return osName.startsWith("windows");
    }

    /**
     * Checks if osName corresponds to Mac OS or similar.
     * Assumes that osName was obtained via
     * System.getProperty("os.name") and is lowercase.
     *
     * @param osName String the name of the operating system
     *        as returned by System.getProperty("os.name") method
     *        and set to lowercase
     * @return boolean <code>true</code> if osName identifies
     *         Mac OS operating system, <code>false</code> otherwise
     */
    private static boolean isMacOS(String osName) {
        return (osName.startsWith("mac os") ||
                osName.startsWith("macos") ||
                osName.startsWith("rhapsody") ||
                osName.startsWith("darwin"));
    }

    /**
     * Checks if osName corresponds to Linux.
     * Assumes osName was obtained via
     * System.getProperty("os.name") and is lowercase.
     *
     * @param osName String the name of the operating system
     *        as returned by System.getProperty("os.name") method
     *        and set to lowercase
     * @return boolean <code>true</code> if osName identifies
     *         Linux operating system, <code>false</code> otherwise
     */
    private static boolean isLinux(String osName) {
        return (osName.startsWith("linux"));
    }

    /**
     * Installs Graphviz dot on Windows or Mac OS. If on Mac OS, dot
     * file permissions are also set to 755. Assumes that the required
     * files can be accessed as resources from the jar file.
     *
     * @param sysDIR String the directory for installing Graphviz dot
     *        in the file system
     * @param resDIR String the directory where to find the resources
     *        of the jar file
     * @param middle String intermediate directory of files in both
     *        file system and jar resources
     * @param dotName String the name of the Graphviz dot executable file
     * @param osName String the name of the operating system as
     *        returned by System.getProperty("os.name") method and
     *        set to lowercase
     * @throws Exception if Graphviz dot files cannot be found in
     *         the proper resources location or Graphviz cannot be
     *         installed in the file system, by failure on either
     *         extracting the files from the jar file or writing
     *         them on the proper file system location
     */
    private static void installDOTWindowsOrMacOS(String sysDIR, String resDIR,
                                          String middle, String dotName,
                                          String osName) throws Exception {
        File file = new File(sysDIR + middle + File.separator + dotName);
        File dir = new File(sysDIR);
        if (file.exists() && file.isFile()){
            return;
        }
        if (!file.exists()) {
            if (!dir.exists()) {
                dir.mkdirs();
            }
            else
            if (!dir.isDirectory()) {
                dir.delete();
                dir.mkdirs();
            }
        }
        dir.copyAllFrom(new File(
            getResourceAbsPath(resDIR +
                               middle.replace(File.separatorChar, '/') +
                               "/" + dotName)
            .replace(middle + File.separator + dotName, "")));
        if (!isWindows(osName)) {
            setFilePermissions(new File(sysDIR + middle +
                                        File.separator + dotName));
        }
    }

    /**
     * Installs Graphviz dot executable on Linux. Additionally,
     * sets dot file permissions to 755.
     *
     * @param sysDIR String the directory for installing Graphviz dot
     *        in the file system
     * @param resDIR String the directory where to find the resources
     *        of the jar file
     * @param middle String intermediate directory of files in both
     *        file system and jar resources
     * @param dotName String the name of the Graphviz dot executable file
     * @throws Exception if Graphviz files cannot be found in their
     *         correct resources paths
     */
/*    private void installDOTLinux(String resDIR, String sysDIR,
                                 String middle, String dotName)
            throws Exception {
        File file = new File(sysDIR + middle + File.separator + dotName);
        File dirBin = new File(sysDIR + File.separator + "bin" + File.separator);
        File dirLib = new File(sysDIR + File.separator + "lib" + File.separator);
        File dirSha = new File(sysDIR + File.separator + "share" + File.separator);
        dirBin.copyAllFrom(new File(
            getResourceAbsPath(resDIR + "/bin/" + dotName)
            .replace(dotName, "")));
        dirLib.copyAllFrom(new File(
            getResourceAbsPath(resDIR + "/lib/libcdt.so.4.0.0")
            .replace("libcdt.so.4.0.0", "")));
        dirSha.copyAllFrom(new File(
            getResourceAbsPath(resDIR + "/share/graphviz/lefty/box.lefty")
            .replace("graphviz/lefty/box.lefty", "")));
        setFilePermissions(new File(
            sysDIR + middle +
            File.separator + dotName));
    }
 */

    /**
     * Sets file permissions of file <code>file</code> to 755.
     *
     * @param file the <code>File</code> to set permissions for
     */
    private static void setFilePermissions(File file) {
        Runtime runtime = Runtime.getRuntime();
        String[] commands = {"chmod", "755", file.getAbsolutePath()};

        try {
            Process process = runtime.exec(commands);
            process.waitFor();
        } catch (Exception ex) {
            System.out.println("Failed to set dot permissions.");
        }
    }

    /**
     * Gets the absolute file system path of a resource.
     *
     * @param path String the path of a resource (in the context
     *        of the jar file)
     * @return String the absolute path of the resource in the file system
     * @throws Exception if resource cannot be found in the given resource path
     */
    private static String getResourceAbsPath(String path) throws Exception {
        ClassLoader cl = BiggestsInstall.class.getClassLoader();
        File file = new File(cl.getResource(path).toURI());
        return file.getAbsolutePath();
    }

    /**
     * Installs GO files.
     *
     * @throws Exception if GO files cannot be found in the
     *         correspondent resource path, cannot be extracted
     *         from their resources location or cannot be written
     *         in the proper file system location
     */
    private void installGOFiles() throws Exception {
        File dir = new File("GO_Files");
        File obo = new File("GO_Files" + File.separator + "gene_ontology.obo");
        File go_zip = new File("go-files.zip");

        if (!go_zip.exists() || obo.exists())
            return;

        if (go_zip.exists()) {
            if (!dir.exists())
                dir.mkdir();
            else
            if (!dir.isDirectory()) {
                dir.delete();
                dir.mkdir();
            }
            dir.copyAllFrom(go_zip);
            go_zip.deleteAll();
        }
    }
}
